@@ -15,14 +15,49 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-"""Functions for working with upstream metadata."""
+"""Functions for working with upstream metadata.
+
+This gathers information about upstreams from various places.
+Each bit of information gathered is wrapped in a UpstreamDatum
+object, which contains the field name.
+
+The fields used here match those in https://wiki.debian.org/UpstreamMetadata
+
+Supported fields:
+- Homepage
+- Name
+- Contact
+- Repository
+- Repository-Browse
+- Bug-Database
+- Bug-Submit
+- Screenshots
+- Archive
+- Security-Contact
+
+Extensions for lintian-brush.
+- X-SourceForge-Project
+- X-Wiki
+- X-Summary
+- X-Description
+- X-License
+- X-Copyright
+
+Supported, but unused.
+- FAQ
+- Donation
+- Documentation
+- Registration
+- Webservice
+"""
 
 import json
+import operator
 import os
 import re
 import socket
 import urllib.error
-from typing import Optional
+from typing import Optional, Union, Iterable
 from urllib.parse import quote, urlparse, urlunparse, urljoin, parse_qs
 from warnings import warn
 
@@ -44,8 +79,10 @@ from lintian_brush import (
 from lintian_brush.vcs import (
     browse_url_from_repo_url,
     plausible_url as plausible_vcs_url,
+    plausible_browse_url as plausible_vcs_browse_url,
     sanitize_url as sanitize_vcs_url,
     is_gitlab_site,
+    determine_browser_url,
     )
 from urllib.request import urlopen, Request
 
@@ -87,31 +124,49 @@ class UpstreamDatum(object):
             self.origin)
 
 
-# See https://wiki.debian.org/UpstreamMetadata
-# Supported fields:
-# - Homepage
-# - Name
-# - Contact
-# - Repository
-# - Repository-Browse
-# - Bug-Database
-# - Bug-Submit
-# - Screenshots
-# - Archive
-# - X-SourceForge-Project
-# - X-Wiki
-# - X-Summary
-# - X-Description
-# - X-License
-# - X-Copyright
+class UpstreamRequirement(object):
+    """Upstream dependency."""
 
-# Supported, but unused.
-# - FAQ
-# - Donation
-# - Documentation
-# - Registration
-# - Security-Contact
-# - Webservice
+    def __init__(self, stage, kind, name, origin=None):
+        self.stage = stage
+        self.kind = kind
+        self.name = name
+        self.origin = origin
+
+    def __str__(self):
+        return "%s Upstream Requirement (%s): %s" % (
+            self.stage, self.kind, self.name)
+
+    def __repr__(self):
+        return "%s(%r, %r, %r, origin=%r)" % (
+            type(self).__name__, self.stage, self.kind, self.name, self.origin)
+
+    def __eq__(self, other):
+        return (
+            isinstance(other, type(self)) and self.stage == other.stage and
+            self.kind == other.kind and self.name == other.name and
+            self.origin == other.origin)
+
+
+class UpstreamOutput(object):
+    """Upstream output."""
+
+    def __init__(self, kind, name, origin=None):
+        self.kind = kind
+        self.name = name
+        self.origin = origin
+
+    def __str__(self):
+        return "%s: %s" % (self.kind, self.name)
+
+    def __repr__(self):
+        return "%s(%r, %r, origin=%r)" % (
+            type(self).__name__, self.kind, self.name, self.origin)
+
+    def __eq__(self, other):
+        return (
+            isinstance(other, type(self)) and self.kind == other.kind and
+            self.name == other.name and self.origin == other.origin)
 
 
 def _load_json_url(http_url: str, timeout: int = DEFAULT_URLLIB_TIMEOUT):
@@ -138,6 +193,23 @@ def get_sf_metadata(project):
         raise NoSuchSourceForgeProject(project)
 
 
+class NoSuchRepologyProject(Exception):
+
+    def __init__(self, project):
+        self.project = project
+
+
+def get_repology_metadata(srcname, repo='debian_unstable'):
+    url = ('https://repology.org/tools/project-by?repo=%s&name_type=srcname'
+           '&target_page=api_v1_project&name=%s' % (repo, srcname))
+    try:
+        return _load_json_url(url)
+    except urllib.error.HTTPError as e:
+        if e.status != 404:
+            raise
+        raise NoSuchRepologyProject(srcname)
+
+
 def guess_repo_from_url(url, net_access=False):
     parsed_url = urlparse(url)
     path_elements = parsed_url.path.strip('/').split('/')
@@ -198,6 +270,8 @@ def guess_repo_from_url(url, net_access=False):
         if parsed_url.path.strip('/').count('/') < 1:
             return None
         parts = parsed_url.path.split('/')
+        if 'issues' in parts:
+            parts = parts[:parts.index('issues')]
         if 'tags' in parts:
             parts = parts[:parts.index('tags')]
         if parts[-1] == '-':
@@ -245,7 +319,8 @@ def known_bad_guess(datum):
     return False
 
 
-def filter_bad_guesses(guessed_items):
+def filter_bad_guesses(
+        guessed_items: Iterable[UpstreamDatum]) -> Iterable[UpstreamDatum]:
     return filter(lambda x: not known_bad_guess(x), guessed_items)
 
 
@@ -261,17 +336,6 @@ def update_from_guesses(upstream_metadata, guessed_items):
     return changed
 
 
-def read_python_pkg_info(path):
-    """Get the metadata from a python setup.py file."""
-    from email.parser import Parser
-    from email.message import Message
-    try:
-        with open(os.path.join(path, 'PKG-INFO'), 'r') as f:
-            return Parser().parse(f)
-    except FileNotFoundError:
-        return Message()
-
-
 def guess_from_debian_rules(path, trust_package):
     from ..rules import Makefile
     mf = Makefile.from_path(path)
@@ -281,13 +345,13 @@ def guess_from_debian_rules(path, trust_package):
         pass
     else:
         yield UpstreamDatum(
-            "Repository", sanitize_vcs_url(upstream_git.decode()), "certain")
+            "Repository", sanitize_vcs_url(upstream_git.decode()), "likely")
     try:
         upstream_url = mf.get_variable(b'DEB_UPSTREAM_URL')
     except KeyError:
         pass
     else:
-        yield UpstreamDatum("X-Download", upstream_url.decode(), "certain")
+        yield UpstreamDatum("X-Download", upstream_url.decode(), "likely")
 
 
 def guess_from_debian_watch(path, trust_package):
@@ -310,19 +374,24 @@ def guess_from_debian_watch(path, trust_package):
             url = w.format_url(package=get_package_name)
             if 'mode=git' in w.options:
                 yield UpstreamDatum(
-                    "Repository", sanitize_vcs_url(url), "confident")
+                    "Repository", sanitize_vcs_url(url), "confident",
+                    origin=path)
                 continue
             if url.startswith('https://') or url.startswith('http://'):
                 repo = guess_repo_from_url(url)
                 if repo:
                     yield UpstreamDatum(
-                        "Repository", sanitize_vcs_url(repo), "likely")
+                        "Repository", sanitize_vcs_url(repo), "likely",
+                        origin=path)
                     continue
             m = re.match('https?://sf.net/([^/]+)', url)
             if m:
-                yield UpstreamDatum("Archive", "SourceForge", "certain")
                 yield UpstreamDatum(
-                    "X-SourceForge-Project", m.group(1), "certain")
+                    "Archive", "SourceForge", "certain",
+                    origin=path)
+                yield UpstreamDatum(
+                    "X-SourceForge-Project", m.group(1), "certain",
+                    origin=path)
                 continue
 
 
@@ -363,6 +432,8 @@ def guess_from_python_metadata(pkg_info):
     if payload.strip() and pkg_info.get_content_type() in (None, 'text/plain'):
         yield UpstreamDatum(
             'X-Description', pkg_info.get_payload(), 'possible')
+    for require in pkg_info.get_all('Requires', []):
+        yield UpstreamRequirement('build', 'python-package', require)
 
 
 def guess_from_pkg_info(path, trust_package):
@@ -411,6 +482,15 @@ def guess_from_setup_py(path, trust_package):
         if url_type in ('Bug Tracker', ):
             yield UpstreamDatum(
                 'Bug-Database', url, 'certain')
+    for require in result.get_requires():
+        yield UpstreamRequirement('build', 'python-package', require)
+    for script in (result.scripts or []):
+        yield UpstreamOutput('binary', os.path.basename(script))
+    entry_points = result.entry_points or {}
+    for script in entry_points.get('console_scripts', []):
+        yield UpstreamOutput('binary', script.split('=')[0])
+    for package in result.packages or []:
+        yield UpstreamOutput('python-package', package)
 
 
 def guess_from_package_json(path, trust_package):
@@ -557,29 +637,75 @@ def guess_from_debian_copyright(path, trust_package):
                 yield UpstreamDatum("Repository", url, 'certain')
 
 
-def guess_from_readme(path, trust_package):
+def url_from_git_clone_command(command):
     import shlex
+    argv = shlex.split(command.decode('utf-8', 'replace'))
+    args = [arg for arg in argv if arg.strip()]
+    i = 0
+    while i < len(args):
+        if not args[i].startswith('-'):
+            i += 1
+            continue
+        if '=' in args[i]:
+            del args[i]
+            continue
+        # arguments that take a parameter
+        if args[i] in ('-b', '--depth', '--branch'):
+            del args[i]
+            del args[i]
+            continue
+        del args[i]
+    try:
+        url = args[2]
+    except IndexError:
+        url = args[0]
+    if plausible_vcs_url(url):
+        return sanitize_vcs_url(url)
+    return None
+
+
+def url_from_fossil_clone_command(command):
+    import shlex
+    argv = shlex.split(command.decode('utf-8', 'replace'))
+    args = [arg for arg in argv if arg.strip()]
+    i = 0
+    while i < len(args):
+        if not args[i].startswith('-'):
+            i += 1
+            continue
+        if '=' in args[i]:
+            del args[i]
+            continue
+        del args[i]
+    try:
+        url = args[2]
+    except IndexError:
+        url = args[0]
+    if plausible_vcs_url(url):
+        return sanitize_vcs_url(url)
+    return None
+
+
+def guess_from_readme(path, trust_package):
     urls = []
     try:
         with open(path, 'rb') as f:
             lines = list(f.readlines())
             for i, line in enumerate(lines):
                 line = line.strip()
-                if line.strip().lstrip(b'$').strip().startswith(b'git clone'):
-                    line = line.strip().lstrip(b'$').strip()
-                    while line.endswith(b'\\'):
-                        line += lines[i+1]
-                        line = line.strip()
+                cmdline = line.strip().lstrip(b'$').strip()
+                if (cmdline.startswith(b'git clone ') or
+                        cmdline.startswith(b'fossil clone ')):
+                    while cmdline.endswith(b'\\'):
+                        cmdline += lines[i+1]
+                        cmdline = cmdline.strip()
                         i += 1
-                    argv = shlex.split(line.decode('utf-8', 'replace'))
-                    args = [arg for arg in argv[2:]
-                            if not arg.startswith('-') and arg.strip()]
-                    try:
-                        url = args[-2]
-                    except IndexError:
-                        url = args[0]
-                    if plausible_vcs_url(url):
-                        urls.append(sanitize_vcs_url(url))
+                    if cmdline.startswith(b'git clone '):
+                        url = url_from_git_clone_command(cmdline)
+                    elif cmdline.startswith(b'fossil clone '):
+                        url = url_from_fossil_clone_command(cmdline)
+                    if url:
+                        urls.append(url)
                 project_re = b'([^/]+)/([^/?.()"#>\\s]*[^-/?.()"#>\\s])'
                 for m in re.finditer(
                         b'https://travis-ci.org/' + project_re, line):
@@ -619,10 +745,9 @@ def guess_from_readme(path, trust_package):
                 for m in re.finditer(
                         b'https://([^/]+)/([^\\s()"#]+)', line):
                     if is_gitlab_site(m.group(1).decode()):
-                        yield UpstreamDatum(
-                            'Repository',
-                            m.group(0).rstrip(b'.').decode().rstrip(),
-                            'possible')
+                        url = m.group(0).rstrip(b'.').decode().rstrip()
+                        repo_url = guess_repo_from_url(url)
+                        yield UpstreamDatum('Repository', repo_url, 'possible')
     except IsADirectoryError:
         pass
 
@@ -707,6 +832,8 @@ def guess_from_meta_yml(path, trust_package):
                 if url:
                     yield UpstreamDatum(
                         'Repository', sanitize_vcs_url(url), 'certain')
+            for require in data.get('requires', []):
+                yield UpstreamRequirement('build', 'perl', require)
 
 
 def guess_from_doap(path, trust_package):
@@ -824,7 +951,7 @@ def guess_from_cabal(path, trust_package=False):
             'certain')
 
 
-def is_email_address(value):
+def is_email_address(value: str) -> bool:
     return '@' in value or ' (at) ' in value
 
 
@@ -952,6 +1079,9 @@ def guess_from_cargo(path, trust_package):
             yield UpstreamDatum('Repository', package['repository'], 'certain')
         if 'version' in package:
             yield UpstreamDatum('X-Version', package['version'], 'confident')
+    for name, details in cargo['dependencies'].items():
+        # TODO(jelmer): Look at details['features'], details['version']
+        yield UpstreamRequirement('build', 'cargo-crate', name)
 
 
 def guess_from_pom_xml(path, trust_package=False):
@@ -989,7 +1119,8 @@ def guess_from_pom_xml(path, trust_package=False):
                 url = url_tag.text.split(':', 2)[2]
             else:
                 url = url_tag.text
-            yield UpstreamDatum('Repository-Browse', url, 'certain')
+            if plausible_vcs_browse_url(url):
+                yield UpstreamDatum('Repository-Browse', url, 'certain')
         connection_tag = scm_tag.find('connection')
         if connection_tag is not None:
             connection = connection_tag.text
@@ -1033,6 +1164,24 @@ def guess_from_git_config(path, trust_package=False):
     # TODO(jelmer): Try origin?
 
 
+def guess_from_get_orig_source(path, trust_package=False):
+    with open(path, 'rb') as f:
+        for line in f:
+            if line.startswith(b'git clone'):
+                url = url_from_git_clone_command(line)
+                if url:
+                    yield UpstreamDatum('Repository', url, 'likely')
+
+
+# https://docs.github.com/en/free-pro-team@latest/github/\
+# managing-security-vulnerabilities/adding-a-security-policy-to-your-repository
+def guess_from_security_md(path, trust_package=False):
+    if path.startswith('./'):
+        path = path[2:]
+    # TODO(jelmer): scan SECURITY.md for email addresses/URLs with instructions
+    yield UpstreamDatum('X-Security-MD', path, 'certain')
+
+
 def _get_guessers(path, trust_package=False):
     CANDIDATES = [
         ('debian/watch', guess_from_debian_watch),
@@ -1050,6 +1199,10 @@ def _get_guessers(path, trust_package=False):
         ('Cargo.toml', guess_from_cargo),
         ('pom.xml', guess_from_pom_xml),
         ('.git/config', guess_from_git_config),
+        ('debian/get-orig-source.sh', guess_from_get_orig_source),
+        ('SECURITY.md', guess_from_security_md),
+        ('.github/SECURITY.md', guess_from_security_md),
+        ('docs/SECURITY.md', guess_from_security_md),
         ]
 
     # Search for something Python-y
@@ -1089,7 +1242,8 @@ def _get_guessers(path, trust_package=False):
         n for n in os.listdir(path)
         if any([n.startswith(p)
                 for p in ['readme', 'README', 'HACKING', 'CONTRIBUTING']])
-        and os.path.splitext(n)[1] not in ('.html', '.pdf', '.xml')]
+        and os.path.splitext(n)[1] not in ('.html', '.pdf', '.xml')
+        and not n.endswith('~')]
     CANDIDATES.extend([(n, guess_from_readme) for n in readme_filenames])
 
     try:
@@ -1103,17 +1257,19 @@ def _get_guessers(path, trust_package=False):
         CANDIDATES.extend(
             [(p, guess_from_debian_patch) for p in debian_patches])
 
-    yield guess_from_environment()
+    yield 'environment', guess_from_environment()
 
     for relpath, guesser in CANDIDATES:
         abspath = os.path.join(path, relpath)
         if not os.path.exists(abspath):
             continue
-        yield guesser(abspath, trust_package=trust_package)
+        yield relpath, guesser(abspath, trust_package=trust_package)
 
 
-def guess_upstream_metadata_items(path, trust_package=False,
-                                  minimum_certainty=None):
+def guess_upstream_metadata_items(
+        path: str, trust_package: bool = False,
+        minimum_certainty: Optional[str] = None
+        ) -> Iterable[UpstreamDatum]:
     """Guess upstream metadata items, in no particular order.
 
     Args:
@@ -1121,14 +1277,23 @@ def guess_upstream_metadata_items(path, trust_package=False,
       trust: Whether to trust the package contents and i.e. run
       executables in it
     Yields:
-      Tuples with (key, value, certainty)
+      UpstreamDatum
     """
+    for entry in guess_upstream_info(path, trust_package=trust_package):
+        if isinstance(entry, UpstreamDatum):
+            if certainty_sufficient(entry.certainty, minimum_certainty):
+                yield entry
+
+
+def guess_upstream_info(
+        path: str, trust_package: bool = False) -> Iterable[
+            Union[UpstreamDatum, UpstreamRequirement, UpstreamOutput]]:
     guessers = _get_guessers(path, trust_package=trust_package)
-    for guesser in guessers:
-        for datum in guesser:
-            if not certainty_sufficient(datum.certainty, minimum_certainty):
-                continue
-            yield datum
+    for name, guesser in guessers:
+        for entry in guesser:
+            if entry.origin is None:
+                entry.origin = name
+            yield entry
 
 
 def guess_upstream_metadata(
@@ -1148,8 +1313,7 @@ def guess_upstream_metadata(
     update_from_guesses(
         upstream_metadata,
         filter_bad_guesses(
-            guess_upstream_metadata_items(
-                path, trust_package=trust_package)))
+            guess_upstream_metadata_items(path, trust_package=trust_package)))
 
     extend_upstream_metadata(
         upstream_metadata, path, net_access=net_access,
@@ -1232,6 +1396,45 @@ def guess_from_sf(sf_project):
             yield 'Repository', url
 
 
+def guess_from_repology(repology_project):
+    try:
+        metadata = get_repology_metadata(repology_project)
+    except socket.timeout:
+        warn('timeout contacting repology, ignoring: %s' % repology_project)
+        return
+
+    fields = {}
+
+    def _add_field(name, value, add):
+        fields.setdefault(name, {})
+        fields[name].setdefault(value, 0)
+        fields[name][value] += add
+
+    for entry in metadata:
+        if entry.get('status') == 'outdated':
+            score = 1
+        else:
+            score = 10
+
+        if 'www' in entry:
+            for www in entry['www']:
+                _add_field('Homepage', www, score)
+
+        if 'licenses' in entry:
+            for license in entry['licenses']:
+                _add_field('X-License', license, score)
+
+        if 'summary' in entry:
+            _add_field('X-Summary', entry['summary'], score)
+
+        if 'downloads' in entry:
+            for download in entry['downloads']:
+                _add_field('X-Download', download, score)
+
+    for field, scores in fields.items():
+        yield field, max(scores, key=operator.itemgetter(1))
+
+
 def extend_from_external_guesser(
         upstream_metadata, guesser_certainty, guesser_fields, guesser):
     if not _possible_fields_missing(
@@ -1244,6 +1447,21 @@ def extend_from_external_guesser(
          for (key, value) in guesser])
 
 
+def extend_from_repology(upstream_metadata, minimum_certainty, source_package):
+    # The set of fields that sf can possibly provide:
+    repology_fields = ['Homepage', 'X-License', 'X-Summary', 'X-Download']
+    certainty = 'confident'
+
+    if certainty_sufficient(certainty, minimum_certainty):
+        # Don't bother talking to repology if we're not
+        # speculating.
+        return
+
+    return extend_from_external_guesser(
+        upstream_metadata, certainty, repology_fields,
+        guess_from_repology(source_package))
+
+
 def extend_from_sf(upstream_metadata, sf_project):
     # The set of fields that sf can possibly provide:
     sf_fields = ['Homepage', 'Name', 'Repository']
@@ -1295,9 +1513,9 @@ def extract_sf_project_name(url):
     m = re.fullmatch('https?://(.*).(sf|sourceforge).net/?', url)
     if m:
         return m.group(1)
-    m = re.match('https://sourceforge.net/projects/([^/]+)', url)
+    m = re.match('https://sourceforge.net/(projects|p)/([^/]+)', url)
     if m:
-        return m.group(1)
+        return m.group(2)
 
 
 def repo_url_from_merge_request_url(url):
@@ -1382,6 +1600,15 @@ def bug_database_url_from_bug_submit_url(url):
             path_elements.pop(-1)
         return urlunparse(
             parsed_url._replace(path='/'.join(path_elements)))
+    if parsed_url.hostname == 'sourceforge.net':
+        if len(path_elements) < 3:
+            return None
+        if path_elements[0] != 'p' or path_elements[2] != 'bugs':
+            return None
+        if len(path_elements) > 3:
+            path_elements.pop(-1)
+        return urlunparse(
+            parsed_url._replace(path='/'.join(path_elements)))
     return None
 
 
@@ -1599,20 +1826,38 @@ def _copy_bug_db_field(upstream_metadata, net_access):
     return ret
 
 
+def _extrapolate_security_contact_from_security_md(
+        upstream_metadata, net_access):
+    repository_url = upstream_metadata['Repository']
+    security_md_path = upstream_metadata['X-Security-MD']
+    vcs_url = unsplit_vcs_url(
+        repository_url.value, None, security_md_path.value)
+    security_url = determine_browser_url(None, vcs_url)
+    if security_url is None:
+        return None
+    return UpstreamDatum(
+        'Security-Contact', security_url,
+        certainty=min_certainty(
+            [repository_url.certainty, security_md_path.certainty]),
+        origin=security_md_path.origin)
+
+
 EXTRAPOLATE_FNS = [
-    ('Homepage', 'Repository', _extrapolate_repository_from_homepage),
-    ('Bugs-Database', 'Bug-Database', _copy_bug_db_field),
-    ('Bug-Database', 'Repository', _extrapolate_repository_from_bug_db),
-    ('Repository', 'Repository-Browse',
+    (['Homepage'], 'Repository', _extrapolate_repository_from_homepage),
+    (['Bugs-Database'], 'Bug-Database', _copy_bug_db_field),
+    (['Bug-Database'], 'Repository', _extrapolate_repository_from_bug_db),
+    (['Repository'], 'Repository-Browse',
      _extrapolate_repository_browse_from_repository),
-    ('Repository-Browse', 'Repository',
+    (['Repository-Browse'], 'Repository',
      _extrapolate_repository_from_repository_browse),
-    ('Repository', 'Bug-Database',
+    (['Repository'], 'Bug-Database',
      _extrapolate_bug_database_from_repository),
-    ('Bug-Database', 'Bug-Submit', _extrapolate_bug_submit_from_bug_db),
-    ('Bug-Submit', 'Bug-Database', _extrapolate_bug_db_from_bug_submit),
-    ('X-Download', 'Repository', _extrapolate_repository_from_download),
-    ('Repository', 'Name', _extrapolate_name_from_repository),
+    (['Bug-Database'], 'Bug-Submit', _extrapolate_bug_submit_from_bug_db),
+    (['Bug-Submit'], 'Bug-Database', _extrapolate_bug_db_from_bug_submit),
+    (['X-Download'], 'Repository', _extrapolate_repository_from_download),
+    (['Repository'], 'Name', _extrapolate_name_from_repository),
+    (['Repository', 'X-Security-MD'],
+     'Security-Contact', _extrapolate_security_contact_from_security_md),
 ]
 
 
@@ -1621,13 +1866,20 @@ def extend_upstream_metadata(upstream_metadata, path, minimum_certainty=None,
                              consult_external_directory=False):
     """Extend a set of upstream metadata.
     """
-    if 'Homepage' in upstream_metadata:
-        project = extract_sf_project_name(upstream_metadata['Homepage'].value)
+    # TODO(jelmer): Use EXTRAPOLATE_FNS mechanism for this?
+    for field in ['Homepage', 'Bug-Database', 'Bug-Submit', 'Repository',
+                  'Repository-Browse', 'X-Download']:
+        if field not in upstream_metadata:
+            continue
+        project = extract_sf_project_name(upstream_metadata[field].value)
         if project:
+            certainty = min_certainty(
+                ['likely', upstream_metadata[field].certainty])
             upstream_metadata['Archive'] = UpstreamDatum(
-                'Archive', 'SourceForge', 'likely')
+                'Archive', 'SourceForge', certainty)
             upstream_metadata['X-SourceForge-Project'] = UpstreamDatum(
-                'X-SourceForge-Project', project, 'likely')
+                'X-SourceForge-Project', project, certainty)
+            break
 
     archive = upstream_metadata.get('Archive')
     if (archive and archive.value == 'SourceForge' and
@@ -1648,20 +1900,29 @@ def extend_upstream_metadata(upstream_metadata, path, minimum_certainty=None,
         else:
             extend_from_lp(upstream_metadata, minimum_certainty, package)
             extend_from_aur(upstream_metadata, minimum_certainty, package)
+            extend_from_repology(upstream_metadata, minimum_certainty, package)
     pecl_url = upstream_metadata.get('X-Pecl-URL')
     if net_access and pecl_url:
         extend_from_pecl(upstream_metadata, pecl_url.value, pecl_url.certainty)
     changed = True
     while changed:
         changed = False
-        for from_field, to_field, fn in EXTRAPOLATE_FNS:
-            try:
-                from_value = upstream_metadata[from_field]
-            except KeyError:
+        for from_fields, to_field, fn in EXTRAPOLATE_FNS:
+            from_certainties = []
+            for from_field in from_fields:
+                try:
+                    from_value = upstream_metadata[from_field]
+                except KeyError:
+                    from_certainties = None
+                    break
+                from_certainties.append(from_value.certainty)
+            if not from_certainties:
+                # Nope
                 continue
+            from_certainty = min_certainty(from_certainties)
             old_to_value = upstream_metadata.get(to_field)
             if old_to_value is not None and (
-                    certainty_to_confidence(from_value.certainty) >=
+                    certainty_to_confidence(from_certainty) >=
                     certainty_to_confidence(old_to_value.certainty)):
                 continue
             result = fn(upstream_metadata, net_access)
@@ -1695,7 +1956,7 @@ def _version_in_tags(version, tag_names):
     return False
 
 
-def probe_upstream_branch_url(url, version=None):
+def probe_upstream_branch_url(url: str, version=None):
     parsed = urlparse(url)
     if parsed.scheme in ('git+ssh', 'ssh', 'bzr+ssh'):
         # Let's not probe anything possibly non-public.
